package org.processmining.xeslite.external;

import java.io.File;
import java.io.IOException;

import org.mapdb.DB;
import org.mapdb.DBMaker;
import org.mapdb.DBMaker.Maker;

final class MapDBDatabase {

	private DB db;
	private File dbFile;

	private boolean useAsyncWriter = true;
	private boolean useCache = true;
	private boolean isTemporary = true;

	public MapDBDatabase() {
		this(null, true);
	}

	public MapDBDatabase(File dbFile) {
		this(dbFile, false);
	}

	public MapDBDatabase(File dbFile, boolean isTemporary) {
		super();
		this.dbFile = dbFile;
		this.isTemporary = isTemporary;
	}
	
	public synchronized DB createDB() throws IOException {
		if (dbFile == null) {
			dbFile = File.createTempFile("xeslite", ".db");
		}
		Maker dbMaker = DBMaker.fileDB(dbFile)
				.closeOnJvmShutdown()				
				.compressionEnable() 
				.transactionDisable()
				.executorEnable()
				.lockSingleEnable()
				.fileMmapCleanerHackEnable() // for beta4
				.fileMmapEnableIfSupported();
		if (isTemporary) {
			dbMaker.deleteFilesAfterClose();
		}
		if (useCache) {
			if (Runtime.getRuntime().maxMemory() >= XFactoryExternalStore.MIN_MEMORY_FOR_CACHING) {
				dbMaker = dbMaker.cacheHashTableEnable(XFactoryExternalStore.DEFAULT_CACHE_SIZE);
			}
		}
		if (useAsyncWriter) {
			dbMaker = dbMaker.asyncWriteEnable().asyncWriteFlushDelay(50).asyncWriteQueueSize(1024 * 8);
		}
		db = dbMaker.make();
		return db;
	}
	
	public DB getDB() {
		return db;
	}

	public void compact() {
		getDB().compact();
	}

	public void close() {
		getDB().close();
	}

}
