/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import java.util.Date;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;

import com.google.common.primitives.Longs;

/**
 * @author F. Mannhardt
 * 
 */
final class XAttributeTimestampExternalImpl extends XAttributeExternalImpl implements XAttributeTimestamp {

	private static final long serialVersionUID = 1L;

	private final long value;

	public XAttributeTimestampExternalImpl(int key, Date value, ExternalStore store) {
		this(key, value, null, store);
	}

	public XAttributeTimestampExternalImpl(int key, Date value, XExtension extension, ExternalStore store) {
		this(key, value.getTime(), extension, store);
	}

	public XAttributeTimestampExternalImpl(int key, long millis, ExternalStore store) {
		this(key, millis, null, store);
	}

	public XAttributeTimestampExternalImpl(int key, long millis, XExtension extension, ExternalStore store) {
		super(key, extension, store);
		this.value = millis;			
	}

	@Override
	public Date getValue() {
		return new Date(value);
	}

	@Override
	public long getValueMillis() {
		return this.value;
	}

	@Override
	public void setValue(Date value) {
		throw new UnsupportedOperationException(IMMUTABLE_ERROR);
	}

	@Override
	public void setValueMillis(long value) {
		throw new UnsupportedOperationException(IMMUTABLE_ERROR);
	}

	@Override
	public String toString() {
		synchronized (FORMATTER) {
			return FORMATTER.format(getValue());
		}
	}

	@Override
	public Object clone() {
		// Creating a 'normal' attribute since the clone is not yet saved in MapDB
		XAttributeTimestamp clone = new XAttributeTimestampImpl(getKey(), value, getExtension());
		if (hasAttributes()) {
			clone.setAttributes(getAttributes());
		}
		return clone;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof XAttributeTimestamp) { // compares types
			XAttributeTimestamp other = (XAttributeTimestamp) obj;
			return super.equals(other) // compares keys
					&& value == other.getValueMillis(); // compares values
		} else {
			return false;
		}
	}

	@Override
	public int compareTo(XAttribute other) {
		if (!(other instanceof XAttributeTimestamp)) {
			throw new ClassCastException();
		}
		int result = super.compareTo(other);
		if (result != 0) {
			return result;
		}
		return Longs.compare(value, ((XAttributeTimestamp) other).getValueMillis());
	}

}
